locals {
  es_cluster_ebs_iops = var.es_cluster_ebs_volume_type == "io1" ? var.es_cluster_ebs_iops : 0
}

data "aws_region" "current" {}

data "aws_caller_identity" "current" {}

resource "aws_elasticsearch_domain" "cluster" {

  domain_name           = var.cluster_domain_name
  elasticsearch_version = var.es_version
  #Required since we enable basic auth
  access_policies = <<POLICY
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": "es:*",
      "Principal": "*",
      "Effect": "Allow",
      "Resource": "${var.aws_arn_prefix}:es:${data.aws_region.current.name}:${data.aws_caller_identity.current.account_id}:domain/${var.cluster_domain_name}/*"
    }
  ]
}
POLICY

  # Required to set this for TF -- it keeps trying to set it to null. (https://github.com/cloudposse/terraform-aws-elasticsearch/issues/81)
  advanced_options = {
    "rest.action.multi.allow_explicit_index" = true
  }

  advanced_security_options {
    enabled                        = true
    internal_user_database_enabled = true
    master_user_options {
      master_user_name     = var.es_cluster_master_user_name
      master_user_password = var.es_cluster_master_user_password
    }
  }

  lifecycle {
    ignore_changes = [
      advanced_security_options,
    ]
  }

  cluster_config {
    instance_type          = var.es_cluster_instance_type
    instance_count         = length(var.subnet_ids) * var.es_cluster_instance_count
    zone_awareness_enabled = true

    dedicated_master_enabled = var.es_cluster_dedicated_master_enabled
    dedicated_master_type    = var.es_cluster_dedicated_master_type
    dedicated_master_count   = var.es_cluster_dedicated_master_count

    warm_enabled = var.es_cluster_warm_enabled
    warm_count   = var.es_cluster_warm_count
    warm_type    = var.es_cluster_warm_type


    zone_awareness_config {
      availability_zone_count = length(var.subnet_ids)
    }
  }

  # https://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-configuration-api.html#es-configuration-api-datatypes-domainendpointoptions
  domain_endpoint_options {
    enforce_https       = true
    tls_security_policy = "Policy-Min-TLS-1-2-2019-07"
  }

  ebs_options {
    ebs_enabled = var.es_cluster_ebs_volume_size > 0 ? true : false
    volume_size = var.es_cluster_ebs_volume_size
    volume_type = var.es_cluster_ebs_volume_type
    iops        = local.es_cluster_ebs_iops
  }

  encrypt_at_rest {
    enabled = true
  }

  # log_publishing_options {}  # TODO Decide on ES logging

  node_to_node_encryption {
    enabled = true
  }

  snapshot_options {
    automated_snapshot_start_hour = var.es_cluster_automated_snapshot_start_hour
  }

  timeouts {
    update = "120m"
  }

  tags = var.tags

  vpc_options {
    subnet_ids         = var.subnet_ids
    security_group_ids = var.security_group_ids
  }
}


